package com.winhc.bigdata.spark.ng.dynamic.agg

import com.winhc.bigdata.spark.implicits.BaseHelper._
import com.winhc.bigdata.spark.implicits.CaseClass2JsonHelper._
import com.winhc.bigdata.spark.ng.dynamic.utils.{BusinessInfoDynamicInfoMap, ChangeContent}
import com.winhc.bigdata.spark.ng.dynamic.{AcrossTabAggHandle, CompanyDynamicRecord}

/**
 * @author: XuJiakai
 * @date: 2021/7/28 17:36
 */
case class BusinessInfo() extends AcrossTabAggHandle {
  /**
   * 需要聚合的维度
   *
   * @return
   */
  override def getTables: Seq[String] = Seq("company", "company_staff")

  /**
   * 聚合的key，相同的key会聚合到一起
   *
   * @return
   */
  override def group_by_key: CompanyDynamicRecord => String = (cdr: CompanyDynamicRecord) => {
    val info = cdr.association_entity_info.filter(_.keyno != null).find(r => r.keyno.length == 32).orNull
    if (info == null) {
      s"${cdr.id}_${cdr.dynamic_info("base").asInstanceOf[BusinessInfoDynamicInfoMap].label}"
    } else {
      s"${info.keyno}_${cdr.dynamic_info("base").asInstanceOf[BusinessInfoDynamicInfoMap].label}"
    }
  }

  /**
   * 聚合处理程序  flat_map
   *
   * @return
   */
  override def group_by_flat_map: Seq[CompanyDynamicRecord] => Seq[CompanyDynamicRecord] = (seq: Seq[CompanyDynamicRecord]) => {
    val contents: Seq[ChangeContent] = seq.flatMap(r => {
      r.dynamic_info("base").asInstanceOf[BusinessInfoDynamicInfoMap].content
    })
    val label = seq.map(r => r.dynamic_info("base").asInstanceOf[BusinessInfoDynamicInfoMap].label).distinct.mkString("，")
    val str = BusinessInfoDynamicInfoMap(label = label, content = contents).toJson()
    Seq(seq(0).customCopy(dynamic_info = str.toAnyMap()))
  }
}
